import { generateScriptableBanner } from "@scriptables/manifest/generateScriptableBanner";
import { normalizeManifest } from "@scriptables/manifest/normalizeManifest";
import { validateManifest } from "@scriptables/manifest/validateManifest";
import { existsSync } from "fs";
import { readFile, writeFile } from "fs/promises";
import { dirname, basename, resolve } from "path";
import { createLogger } from "./logger.js";
import { isMatch, normalizeAndResolvePath, waitForFile } from "./utils.js";
const DEFAULT_MANIFEST_EXTENSIONS = [".manifest.json", ".manifest", ".json"];
function scriptableBanner(options = {}) {
    const { manifestExtensions = DEFAULT_MANIFEST_EXTENSIONS, warnOnMissingManifest = true, warnOnInvalidManifest = true, warnOnMissingEntry = true, allowExtraManifestKeys = true, getManifestPath, patterns = ["*.{ts,tsx,js,jsx}"] } = options;
    const logger = createLogger("scriptable-banner");
    return {
        name: "scriptable-banner",
        setup(build) {
            let manifestCache = /* @__PURE__ */ new Map();
            if (!build.initialOptions.metafile) {
                logger.warn("Warning: \"metafile\" option was not enabled. Automatically setting \"metafile: true\" to allow scriptable plugin to modify output files.");
                build.initialOptions.metafile = true;
            }
            build.onStart(() => {
                manifestCache.clear();
            });
            build.onResolve({ filter: /.*/ }, async (args) => {
                if (args.kind !== "entry-point" || !isMatch(args.path, patterns)) {
                    return null;
                }
                const normalizedPath = normalizeAndResolvePath(args.path);
                if (manifestCache.has(normalizedPath)) {
                    return null;
                }
                const dir = dirname(args.path);
                const baseFileName = basename(args.path).replace(/\.[tj]sx?$/, "");
                const manifestPath = getManifestPath ? getManifestPath(args.path) : manifestExtensions.map((ext) => resolve(dir, `${baseFileName}${ext}`)).find(existsSync);
                if (manifestPath) {
                    try {
                        const manifestContent = await readFile(manifestPath, "utf8");
                        const manifest = normalizeManifest(JSON.parse(manifestContent));
                        try {
                            validateManifest(manifest, allowExtraManifestKeys);
                            manifestCache.set(normalizedPath, manifest);
                        }
                        catch (error) {
                            if (warnOnInvalidManifest && error instanceof Error) {
                                logger.warn(`Invalid manifest file: ${manifestPath} - ${error.message}`, error);
                            }
                        }
                    }
                    catch (error) {
                        logger.warn(`Failed to read manifest file: ${manifestPath}`, error);
                    }
                }
                else if (warnOnMissingManifest) {
                    logger.warn(`No manifest file found for entry point: ${args.path}`);
                }
                return null;
            });
            build.onEnd(async (result) => {
                if (!result.metafile || Object.keys(result.metafile.outputs).length === 0) {
                    logger.warn("Skipping banner addition because \"metafile\" is not available or contains no outputs. Ensure \"metafile: true\" is set in esbuild options if write is enabled.");
                    return;
                }
                const outputs = result.metafile.outputs;
                for (const outputPath of Object.keys(outputs)) {
                    const outputMeta = outputs[outputPath];
                    const entryPoint = outputMeta?.entryPoint ?? "";
                    const normalizedEntryPath = entryPoint ? normalizeAndResolvePath(entryPoint) : "";
                    if (!normalizedEntryPath || !manifestCache.has(normalizedEntryPath)) {
                        if (warnOnMissingEntry) {
                            logger.warn(`No matching entry point found for output file: ${outputPath}`);
                        }
                        continue;
                    }
                    try {
                        const manifest = manifestCache.get(normalizedEntryPath) || {};
                        const banner = generateScriptableBanner(manifest);
                        if (result.outputFiles) {
                            const outputFile = result.outputFiles.find((file) => file.path === outputPath || file.path === "<stdout>");
                            if (outputFile) {
                                outputFile.contents = Buffer.from(banner + outputFile.text.trimEnd());
                            }
                        }
                        else {
                            try {
                                await waitForFile(outputPath);
                                const content = await readFile(outputPath, "utf-8");
                                const updatedContent = banner + content.trimEnd();
                                await writeFile(outputPath, updatedContent, "utf8");
                            }
                            catch (error) {
                                logger.error(`Failed to process file ${outputPath}: ${error instanceof Error ? error.message : String(error)}`);
                            }
                        }
                    }
                    catch (error) {
                        logger.error(`Failed to process output file: ${outputPath}`, error);
                    }
                }
            });
        }
    };
}
export { scriptableBanner };
//# sourceMappingURL=scriptable-banner.js.map
//# sourceMappingURL=scriptable-banner.js.map
